######################################
#Load libraries
library(SSDM)
library(sp)
library(raster)

######    USER PARAMETERS - MODIFY THEM ACCORDING TO YOUR NEEDS ####################

#WORKING FOLDER
setwd("D:/00tmp/WorldClim")

#SET IN "VARIABLESFOLDER" THE FOLDER THAT CONTAINS THE ENVIRONMENTAL VARIABLES
#TO BE USED, IN ANY RASTER FORMAT ACCEPTED BY "raster" PACKAGE (ASC, GEOTIFF...)
VARIABLESFOLDER <- "D:/00tmp/WorldClim/BIO2000"
  
#SET IN "OUTPUTFOLDER" THE OUTPUT FOLDER WHERE RESULT WILL BE SAVED
OUTPUTFOLDER<-"D:/00tmp/Virtuals/SSDM"

#SET IN "CSVSTYLE" 
#     S-> TO IMPORT DATA USING SPANISH-LIKE CSV FORMAT
#     E-> TO IMPORT DATA USING ENGLISH-LIKE CSV FORMAT
CSVSTYLE="S"

#SET IN "OCCURRENCEFILE" THE FILE THAT CONTAINS THE OCCURRENCES TO BE USED TO
#CALCULATE THE ENSEMBLE MODELS. THE FILE MAY CONTAIN OCCURRENCES FOR SEVERAL SPECIES.
#THE EXPECTED FORMAT IS A CSV WITH 3 COLUMNS:  
#                  Species name ,	Longitude,	Latitude
OCCURRENCEFILE<-"D:/00tmp/virtuals/VirtualSpeciesOccurrences.CSV"

#SET IN "AREA" THE RECTANGULAR EXTENT TO WORK WITH.
#TO WORK WITH THE WHOLE EXTENT COVERED BY THE VARIABLES (I.E. THE WORLD)
#JUST COMMENT THIS LINE
AREA <- extent(-84,-33,-58,13) ##extent example that limits to South America

#SET in "MODELS" THE MODELS TO BE USED IN THE ENSEMBLE MODELLING (see help for "ensemble_modelling" for details)
MODELS<-c('all')
#'all' calls all the following algorithms. 
# Generalized linear model (GLM)
# Generalized additive model (GAM)
# Multivariate adaptive regression splines (MARS)
# Generalized boosted regressions model (GBM)
# Classification tree analysis (CTA)
# Random forest (RF)
# Maximum entropy (MAXENT)
# Artificial neural network (ANN)
# Support vector machines (SVM)

#SET in "THRESHOLD" THE threshold value between 0 and 1  associated with the metric(s) used to compute the model selection.
#See "ensemble.thresh" in the  help for "ensemble_modelling" for details
THRESHOLD<-0.6

################################################################


simpleensemblemodelsfun<- function()
{
  dir.create(OUTPUTFOLDER, showWarnings = FALSE)
  
  ##########################
  #Environmental Variables
  EnvVars<-stack()
  files<-list.files((path=VARIABLESFOLDER), ignore.case=TRUE, full.names = TRUE)
  index<-1
  for (fileIn in files){
    print(paste("Loading variable ",index,"->", fileIn))  
    r <- raster(fileIn)
    if (exists("AREA"))
    {
      r<-crop(r, AREA)
    }
    index<-index+1
    EnvVars <- stack(r)    
  }
  
  #Ocurrences
  if (CSVSTYLE=="S")
  {
    Occurrences<-read.csv2(file = OCCURRENCEFILE, header = TRUE)
  }else
  {
    Occurrences<-read.csv(file = OCCURRENCEFILE, header = TRUE)  
  }
  
  ##########################
  
  
  #list of species
  Species<-unique(Occurrences$Species)
  
  outputModelPath<-paste0(OUTPUTFOLDER,"/tmp")
  unlink(outputModelPath, recursive = TRUE)
  dir.create(outputModelPath, showWarnings = FALSE)
  
  
  
  for(speciesName in Species){
  
    ##Ensemble SDM building
    print(paste("Evaluating------------>", speciesName))
    SpeciesOccurrences<-Occurrences[Occurrences$Species==speciesName,]
    
    ESDM <- ensemble_modelling(MODELS, SpeciesOccurrences, EnvVars, rep = 1, 
        Xcol = 'Longitude', Ycol = 'Latitude',
        ensemble.thresh = c(THRESHOLD),name = speciesName, save = TRUE, 
        path = outputModelPath, verbose = FALSE)
    
    if (is.null(ESDM))
    {
      print(paste(speciesName, "Error: Cannot generate model with current params. Try lowering THRESHOLD param"));
    }
    else
    {
      #copy suitability raster to the OUTPUTFOLDER and convert it to ASC format
      suitabFile<-paste0(outputModelPath,"/",speciesName,"/Rasters/Probability.tif")
      r<-raster(suitabFile)
      suitabFile<-paste0(OUTPUTFOLDER,"/",speciesName,".asc")
      writeRaster(r,filename = suitabFile, overwrite = TRUE)
    }
  
  }
  
  print("ENDED")
}

simpleensemblemodelsfun()