#Load libraries
library(raster, quietly = T)
library(rgdal)
library(virtualspecies)
library(dismo)

######    USER PARAMETERS - MODIFY THEM ACCORDING TO YOUR NEEDS ####################

#WORKING FOLDER
setwd("D:/00tmp/WorldClim")

#SET IN "VARIABLESFOLDER" THE FOLDER THAT CONTAINS THE ENVIRONMENTAL VARIABLES
#TO BE USED, IN ANY RASTER FORMAT ACCEPTED BY "raster" PACKAGE (ASC, GEOTIFF...)
VARIABLESFOLDER<-"D:/00tmp/WorldClim/BIO2000"  

#SET IN "OUTPUTFOLDER" THE OUTPUT FOLDER WHERE RESULT WILL BE SAVED
OUTPUTFOLDER<-"D:/00tmp/virtuals"

#SET IN "CSVSTYLE" 
#     S-> TO EXPORT RESULTS USING SPANISH-LIKE CSV FORMAT
#     E-> TO EXPORT RESULTS USING ENGLISH-LIKE CSV FORMAT
CSVSTYLE="S"

#SET IN "AREA" THE RECTANGULAR EXTENT TO WORK WITH.
#TO WORK WITH THE WHOLE EXTENT COVERED BY THE VARIABLES (I.E. THE WORLD)
#JUST COMMENT THIS LINE
AREA <- extent(-84,-33,-58,13) ##extent example that limits to South America

#SET IN "NUMSPECIES" THE NUMBER OF VIRTUAL SPECIES TO BE GENERATED
NUMSPECIES<-5


#SET IN "NUMVARSBYSPECIES" THE NUMBER OF VARIABLES TO BE USED TO GENERATE
#A VIRTUAL SPECIES. THOSE VARIABLES WILL BE RANDOMLY SELECTED FOR EACH SPECIES
#AMONG THE AVAILABLE VARIABLES.
NUMVARSBYSPECIES<-6


#SET IN "MINPREVALENCE" and "MAXPREVALENCE" THE MINIMUM/MAXIMUM PREVALENCE 
#OF THE SPECIES TO BE GENERATED. THE PREVALENCE WILL BE RANDOMLY CHOOSEN 
#BETWEEN THOSE LIMITS FOR EACH SPECIES
MINPREVALENCE<-0.1
MAXPREVALENCE<-0.9


#SET IN "MINOCCURRENCES" and "MAXOCCURRENCES" THE MINIMUM/MAXIMUM NUMBER 
#OF OCCURRENCES TO BE GENERATED FOR EACH SPECIES. THE NUMBER OF OCCURRENCES
#WILL BE RANDOMLY CHOOSEN BETWEEN THOSE LIMITS FOR EACH SPECIES
MINOCCURRENCES<-200 
MAXOCCURRENCES<-500

#SET IN "ALLOWDUPLICATES" IF  DUPLICATE OCURRENCES ARE
#ALLOWED/NOT ALLOWED
ALLOWDUPLICATES<-FALSE

#SET IN "USESUITABILITYASSAMPLINGBIAS" IF THE SUITABILITY SHOULD
#BE USED TO BIAS SAMPLING. IF TRUE, SPECIES SUITABILITY WILL BE USED
#TO INCREASE THE LIKEHOOD OF SAMPLING IN AREAS WITH HIGH  SUITABILITY.
#IF FALSE, NO BIAS WILL BE USED.
USESUITABILITYASSAMPLINGBIAS<-TRUE

#SET IN "REESCALEVARS" TRUE to RESCALE ALL ENVIRONMENTAL VARIABLES
#TO 0-1. FALSE OTHERWISE. AS RESPONSE FUNCTIONS SUCH AS "dnorm" (normal)
#WILL BE USED EXPECTING A [0-1] RANGE IT IS RECOMMEND TO SET IT TO TRUE
RESCALEVARS<-TRUE

##################################################

simplevirtualspeciesfun<- function()
{
  
  if (NUMSPECIES<=0)
  {
    stop("NUMSPECIES should be >0")
  }
  
  if (NUMVARSBYSPECIES<=0)
  {
    stop("NUMVARSBYSPECIES should be >0")
  }
  
  if (MAXOCCURRENCES<=0)
  {
    stop("MAXOCCURRENCES should be >0")
  }  
  
  if (MAXOCCURRENCES<MINOCCURRENCES)
  {
    stop("MAXOCCURRENCES should be >MINOCCURRENCES")
  }    
  
  if (MINPREVALENCE<0 || MINPREVALENCE>MAXPREVALENCE)
  {
    stop("MINPREVALENCE should be >=0 and <MAXPREVALENCE")
  }  
  
  if (MAXPREVALENCE<0 || MAXPREVALENCE>1)
  {
    stop("MAXPREVALENCE should be <1 and >MINPREVALENCE")
  }  
  
  dir.create(OUTPUTFOLDER, showWarnings = FALSE)
  suitabRasteroutputfolder<-paste0(OUTPUTFOLDER,"/Suitability")
  PARasteroutputfolder<-paste0(OUTPUTFOLDER,"/Presence")
  dir.create(suitabRasteroutputfolder, showWarnings = FALSE)
  dir.create(PARasteroutputfolder, showWarnings = FALSE)
  
  OccurrencesFile <-  paste0(OUTPUTFOLDER,"/","VirtualSpeciesOccurrences",".CSV")
  ReportFile <-  paste0(OUTPUTFOLDER,"/","Report",".CSV")
  
  files<-vector()
  #create a list of all variables
  folderlist<-list.files((path=VARIABLESFOLDER), ignore.case=TRUE, full.names = TRUE)
  files<-c(files, folderlist)
  
  if (NUMVARSBYSPECIES>length(files))
  {
    stop("NUMVARSBYSPECIES should be less or equal to the number of available variables")
  }
  
  VarNames<-paste0("V",seq(from=1, to=100, by=1))
  
  
  #we create a random matrix that contains NUMVARSBYSPECIES values for each species
  #these values will be used as indexes in the function of the species
  
  varsPorEspecie<-replicate(NUMSPECIES,sample.int(length(files), 
    size=NUMVARSBYSPECIES, replace=F))
  
  #We create another random matrix that will contain 2 values per species,
  #to use as parameters of the function of each species 
  coefPorEspecie<-replicate(NUMSPECIES,sample(1:100, 
    size=2*NUMVARSBYSPECIES, replace=F)/100)
  
  SpeciesOccurrences<-data.frame()
  
  nVar <- length(files)
  index<-1
  for (fileIn in files){
    print(paste("Loading variable ",index,"->", fileIn))  
    r <- raster(fileIn)
    if (exists("AREA"))
    {
      r<-crop(r, AREA)
    }
    if (RESCALEVARS==TRUE)
    {
      ##reescale variable to [0-1]
      r.min = cellStats(r, "min")
      r.max = cellStats(r, "max")
      r<- ((r - r.min) / (r.max - r.min))    
    }
    index<-index+1
    if (exists("stackVars")){
      stackVars <- addLayer(stackVars,r)
    }else{
      stackVars <- stack(r)    
    }
  }
  
  numespecie<-1
  
  report<-vector()
  
  ##GENERATION OF SPECIES
  for(numespecie in 1:NUMSPECIES){
    
    nombreEspecie<-paste0("Virtual species",numespecie)
    
    #select only the variables that correspond to the species
    if (is.null(dim(varsPorEspecie)))
    {
      datosVar<-subset(stackVars,varsPorEspecie[numespecie])
    } else
    {
      datosVar<-subset(stackVars,varsPorEspecie[,numespecie])
    }
    
    info<-c(nombreEspecie,names(datosVar))
    print(info)
    
    report<-rbind(report, info)
  
    names(datosVar)<-VarNames[1:NUMVARSBYSPECIES]
  
    print(paste(nombreEspecie,"Coeficients:"))
    print(coefPorEspecie[,numespecie])       
  
    #GENERATE THE FUNCTIONS AND PARAMETERS THAT GENERATES THE SPECIES
    generatespecies<-function()
    {
      rnd<-1
      parameters=list()
      for(numvar in 1:NUMVARSBYSPECIES){
        index<-(numvar*2)-1
        #select randomly a function to use with the var
        functionselector=sample(3,1)
        switch(functionselector,
               {vfunc <- formatFunctions( x=NULL, Var=c(fun = 'logisticFun', alpha = coefPorEspecie[index,numespecie]*rnd,beta = coefPorEspecie[index+1,numespecie]*rnd))},
               {vfunc <- formatFunctions( x=NULL, Var=c(fun = 'linearFun', a = coefPorEspecie[index,numespecie]*rnd,b = coefPorEspecie[index+1,numespecie]*rnd))      },
               {vfunc <- formatFunctions( x=NULL, Var=c(fun = 'dnorm', mean= coefPorEspecie[index,numespecie]*rnd,sd = coefPorEspecie[index+1,numespecie*rnd]))      }
        )
        parameters[[paste0("V",numvar)]]=vfunc[["Var"]]
        #info<-c(paste("V",numvar),vfunc$Var$fun, vfunc$Var$args)
      }  
      
    
      #we generate random coefs for each variable
      varcoefs<-runif(NUMVARSBYSPECIES, max=0.2)
      #we scale them to sum up 1
      varcoefs<-varcoefs/sum(varcoefs)
    
      ##Now we define a string with the formula that combines the variables:
      
      formulah <-""
      for(numvar in 1:NUMVARSBYSPECIES){
        formulah <- paste0(formulah,"(",varcoefs[numvar],"* V", numvar ,")")
        if (numvar<NUMVARSBYSPECIES)
        {
          formulah <- paste0(formulah," + ")
        }
      }
      
      prevalence<-runif(1, MINPREVALENCE, MAXPREVALENCE)
      ##We generate the virtual species
      virtualSP <- generateSpFromFun(datosVar,parameters,formula = formulah, plot = FALSE)
      virtualPA<-convertToPA(virtualSP,  species.prevalence=prevalence)      
      return(virtualPA)
    }##END generatespecies

    virtualPA<-generatespecies()
    suitabRaster<-virtualPA$suitab.raster

    ##To save as an ASC file
    fileSuitab <- paste0(suitabRasteroutputfolder,"/",nombreEspecie,"_suitab.asc")
    writeRaster(suitabRaster,filename = fileSuitab, overwrite = TRUE)
    filePA <- paste0(PARasteroutputfolder,"/",nombreEspecie,"_presence.asc")    
    writeRaster(virtualPA$pa.raster,filename = filePA, overwrite = TRUE)

    ##To select randomly
    numOccurrences<-round(runif(1, min=MINOCCURRENCES, max=MAXOCCURRENCES))
    if (USESUITABILITYASSAMPLINGBIAS)
    {
      samples<-sampleOccurrences(virtualPA, numOccurrences, plot = FALSE,
          bias="manual", weights=suitabRaster, replacement = ALLOWDUPLICATES)
    }
    else{
      samples<-sampleOccurrences(virtualPA, numOccurrences, plot = FALSE,
                                 replacement = ALLOWDUPLICATES)      
      }
    occurrences<- samples$sample.points
    occurrences<-occurrences[,c(1,2)]
    occurrences<-cbind(nombreEspecie, occurrences) 
    colnames(occurrences) <- c("Species","Longitude","Latitude")
    SpeciesOccurrences<-rbind(SpeciesOccurrences,occurrences)
    print(paste("END ",nombreEspecie))
  }## END for(numespecie in 1:NUMSPECIES)
  
  #To save as CSV
  if (CSVSTYLE=="S")
  {
    write.csv2(SpeciesOccurrences, file = OccurrencesFile, row.names=FALSE)      
  }else
  {
    write.csv(SpeciesOccurrences, file = OccurrencesFile, row.names=FALSE) 
  }
  report<-as.data.frame(report)
  colnames(report) <- c("Species",VarNames[1:NUMVARSBYSPECIES])
  if (CSVSTYLE=="S")
  {
    write.csv2(report, file = ReportFile, row.names=FALSE)      
  }else{
    write.csv(report, file = ReportFile, row.names=FALSE)      
  }
  
  print("END")
}

simplevirtualspeciesfun()
